import hashlib
import math
import sys
import os

# Helper Functions.
def pad(s):
    s += "0"
    return s + "1"*(8 - len(s)%8)

def unpad(s):
    return s[:s.rfind("0")]

def encode(s):
    return int("".join([('%02x' % ord(c)) for c in s]), 16)

def decode(i):
    hi = '%016x' % i
    res = ""
    for i in xrange(len(hi)/2):
        res += chr(int(hi[2*i:2*(i+1)], 16))
    return res
    
def inner_encr(s, k):
    s = pad(s)
    blcks = [s[8*i:8*(i+1)] for i in xrange(0, int(math.ceil(float(len(s))/8)))]
    x = k
    res = ""
    for b in blcks:
        x = int(hashlib.sha1(str(x)).hexdigest(), 16)
        bi = encode(b)
        res += '%020x ' % (x ^ bi)
    return res

def inner_decr(s, k):
    blcks = s.split()
    x = k
    res = ""
    for b in blcks:
        x = int(hashlib.sha1(str(x)).hexdigest(), 16)
        res += decode(int(b, 16) ^ x)
    return unpad(res)

def check_key(k):
    return len(k) == 14 and all((c>='a' and c<='z') for c in k)

def encrypt(s, k):
    assert check_key(k)
    return inner_encr(inner_encr(s, k[:7]), k[7:])

def decrypt(s, k):
    assert check_key(k)
    return inner_decr(inner_decr(s, k[7:]), k[:7])

# Main Program.
if len(sys.argv) != 4:
    print "Usage: python", sys.argv[0], "key textfile command"
    sys.exit(1)
elif not check_key(sys.argv[1]):
    print "Key", sys.argv[1], "invalid. Has to lowercase and of length 14."
    sys.exit(1)
elif not os.path.exists(sys.argv[2]):
    print "File", sys.argv[2], "does not exist."
    sys.exit(1)
elif sys.argv[3] != "encrypt" and sys.argv[3] != "decrypt":
    print "Command", sys.argv[3], "not one of 'encrypt', 'decrypt'."
    sys.exit(1)

key=sys.argv[1]
f=open(sys.argv[2], "r")
msg=""
for line in f:
     msg += line

if not all(ord(c) < 256 for c in msg):
    print "File", sys.argv[2], "not valid."

if sys.argv[3] == "encrypt":
    print encrypt(msg, key)
else:
    print decrypt(msg, key)
